@@ -20,6 +20,7 @@ from typing import (
     Any,
     Callable,
     Collection,
+    List,
     Mapping,
     NamedTuple,
     Optional,
@@ -105,6 +106,8 @@ class Token(str):
 
 UNSET = Token("UNSET")
 
+debug_var_tip = "(tip: set GRAPHTIK_DEBUG envvar to view Op details in print-outs)"
+
 
 def first_solid(*tristates, default=None):
     """Utility combining multiple tri-state booleans."""
@@ -418,7 +421,7 @@ class PlotArgs(NamedTuple):
     #: the list of output names .
     outputs: Collection = None
     #: Contains the computed results, which might be different from :attr:`plottable`.
-    solution: "graphtik.network.Solution" = None
+    solution: "graphtik.planning.Solution" = None
     #: Either a mapping of node-names to dot(``.``)-separated cluster-names, or
     #: false/true to enable :term:`plotter`'s default clustering of nodes based
     #: on their dot-separated name parts.
@@ -488,14 +491,17 @@ class PlotArgs(NamedTuple):
 ## Defined here, to avoid subclasses importing `plot` module.
 class Plottable(abc.ABC):
     """
-    Classes wishing to plot their graphs should inherit this and ...
+    :term:`plottable` capabilities and graph props for all major classes of the project.
 
+    Classes wishing to plot their graphs should inherit this and
     implement property ``plot`` to return a "partial" callable that somehow
     ends up calling  :func:`.plot.render_pydot()` with the `graph` or any other
     args bound appropriately.
     The purpose is to avoid copying this function & documentation here around.
     """
 
+    graph: "networkx.Graph"
+
     def plot(
         self,
         filename: Union[str, bool, int] = None,
@@ -508,7 +514,7 @@ class Plottable(abc.ABC):
         steps=None,
         inputs=None,
         outputs=None,
-        solution: "graphtik.network.Solution" = None,
+        solution: "graphtik.planning.Solution" = None,
         clusters: Mapping = None,
         jupyter_render: Union[None, Mapping, str] = None,
     ) -> "pydot.Dot":
@@ -676,7 +682,7 @@ class Plottable(abc.ABC):
         **Examples:**
 
             >>> from graphtik import compose, operation
-            >>> from graphtik.modifiers import optional
+            >>> from graphtik.modifier import optional
             >>> from operator import add
 
             >>> pipeline = compose("pipeline",
@@ -705,7 +711,7 @@ class Plottable(abc.ABC):
             digraph pipeline {
             fontname=italic;
             label=<pipeline>;
-            <a> [fillcolor=wheat, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
+            <a> [fillcolor=wheat, label=a, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
             ...
 
         .. graphtik::
@@ -785,6 +791,40 @@ class Plottable(abc.ABC):
           not to override user args.
         """
 
+    @property
+    def ops(self) -> List["Operation"]:
+        """A new list with all :term:`operation`\\s contained in the :term:`network`."""
+        from .planning import yield_ops
+
+        return list(yield_ops(self.graph))
+
+    @property
+    def data(self) -> List[str]:
+        """A new list with all :term:`operation`\\s contained in the :term:`network`."""
+        from .planning import yield_datanodes
+
+        return list(yield_datanodes(self.graph))
+
+    def find_ops(self, predicate) -> List["Operation"]:
+        """
+        Scan operation nodes and fetch those satisfying `predicate`.
+
+        :param predicate:
+            the :term:`node predicate` is a 2-argument callable(op, node-data)
+            that should return true for nodes to include.
+        """
+        return [
+            n
+            for n, d in self.graph.nodes.data(True)
+            if isinstance(n, Operation) and predicate(n, d)
+        ]
+
+    def find_op_by_name(self, name) -> Union["Operation", None]:
+        """Fetch the 1st operation named with the given `name`."""
+        for n in self.ops:
+            if n.name == name:
+                return n
+
 
 class RenArgs(NamedTuple):
     """
@@ -792,7 +832,17 @@ class RenArgs(NamedTuple):
     """
 
     #: what is currently being renamed,
-    #: one of the string: ``(op | needs | provides | aliases)``
+    #: one of the string::
+    #:
+    #:     op
+    #:     need.jsonpart
+    #:     need
+    #:     provide.jsonpart
+    #:     provide
+    #:     alias.jsonpart
+    #:     alias
+    #:
+    #: Any :term:`jsonp` parts are renamed prior to the full path (as ordered above).
     typ: str
     #: the operation currently being processed
     op: "Operation"
@@ -846,66 +896,104 @@ class Operation(Plottable, abc.ABC):
             all data are extracted 1st from this kw, falling back on the operation's
             attributes, and it is modified in-place
 
-        For the other 2 params, see :meth:`.FunctionalOperation.withset()`.
+        For the other 2 params, see :meth:`.FnOp.withset()`.
 
         :raise ValueError:
             - if a `renamer` was neither dict nor callable
             - if a `renamer` dict contained a non-string value,
         """
+        from .modifier import dep_renamed
+
+        def with_errors_logged(fn, ren_args: RenArgs) -> str:
+            """Wrap `fn` to log its errors without touching ex, for debug aid."""
+            ok = False
+            try:
+                ret = fn(ren_args)
+                ok = True
+                return ret
+            finally:
+                if not ok:
+                    log.warning("Failed to rename %s", ren_args)
 
         def rename_driver(ren_args: RenArgs) -> str:
             """Handle dicts, callables and non-string names as true/false."""
-            from .modifiers import dep_renamed
 
-            ok = False
-            try:
-                new_name = old_name = ren_args.name
-                if isinstance(renamer, cabc.Mapping):
-                    dst = renamer.get(old_name)
-                    if callable(dst) or (dst and isinstance(dst, str)):
-                        new_name = dep_renamed(old_name, dst)
-                elif callable(renamer):
-                    dst = renamer(ren_args)
-                    if dst and isinstance(dst, str):
-                        new_name = dst
-                    # A falsy means don't touch the node.
-                else:
-                    raise AssertionError(
-                        f"Invalid `renamer` {renamer!r} should have been caught earlier."
+            new_name = old_name = ren_args.name
+            if isinstance(renamer, cabc.Mapping):
+                dst = renamer.get(old_name)
+                if callable(dst) or (dst and isinstance(dst, str)):
+                    new_name = dep_renamed(old_name, dst)
+            elif callable(renamer):
+                dst = renamer(ren_args)
+                if dst and isinstance(dst, str):
+                    new_name = dst
+                # A falsy means don't touch the node.
+            else:
+                raise AssertionError(
+                    f"Invalid `renamer` {renamer!r} should have been caught earlier."
+                )
+
+            if (not new_name and old_name) or not isinstance(new_name, str):
+                raise ValueError(
+                    f"Must rename {old_name!r} into a non-empty string, got {new_name!r}!"
+                )
+
+            return new_name
+
+        def rename_subdocs(ren_args):
+            parts = getattr(ren_args.name, "_jsonp", None)
+            if parts:  # assuming deps here have been jsonpized earlier.
+                path = "/".join(
+                    rename_driver(
+                        ren_args._replace(typ=ren_args.typ + ".jsonpart", name=p)
                     )
-
-                if not new_name or not isinstance(new_name, str):
-                    raise ValueError(
-                        f"Must rename {old_name!r} into a non-empty string, got {new_name!r}!"
-                    )
-
-                ok = True
-                return new_name
-            finally:
-                if not ok:
-                    # Debug aid without touching ex.
-                    log.warning("Failed to rename %s", ren_args)
+                    for p in parts
+                )
+                ren_args = ren_args._replace(name=dep_renamed(ren_args.name, path))
+            return rename_driver(ren_args)
 
         ren_args = RenArgs(None, self, None)
 
-        kw["name"] = rename_driver(
-            ren_args._replace(typ="op", name=kw.get("name", self.name))
+        kw["name"] = with_errors_logged(
+            rename_driver, ren_args._replace(typ="op", name=kw.get("name", self.name))
         )
-        ren_args = ren_args._replace(typ="needs")
+        ren_args = ren_args._replace(typ="need")
         kw["needs"] = [
-            rename_driver(ren_args._replace(name=n))
+            with_errors_logged(rename_subdocs, ren_args._replace(name=n))
             for n in kw.get("needs", self.needs)
         ]
-        ren_args = ren_args._replace(typ="provides")
+        ren_args = ren_args._replace(typ="provide")
         # Store renamed `provides` as map, used for `aliases` below.
         renamed_provides = {
-            n: rename_driver(ren_args._replace(name=n))
+            n: with_errors_logged(rename_subdocs, ren_args._replace(name=n))
             for n in kw.get("provides", self.provides)
         }
         kw["provides"] = list(renamed_provides.values())
         if hasattr(self, "aliases"):
-            ren_args = ren_args._replace(typ="aliases")
+            ren_args = ren_args._replace(typ="alias")
             kw["aliases"] = [
-                (renamed_provides[k], rename_driver(ren_args._replace(name=v)),)
+                (
+                    renamed_provides[k],
+                    with_errors_logged(rename_subdocs, ren_args._replace(name=v)),
+                )
                 for k, v in kw.get("aliases", self.aliases)  # pylint: disable=no-member
             ]
+
+    def prepare_plot_args(self, plot_args: PlotArgs) -> PlotArgs:
+        """Delegate to a provisional network with a single op . """
+        from .pipeline import compose
+        from .plot import graphviz_html_string
+
+        is_user_label = bool(plot_args.graph and plot_args.graph.get("graphviz.label"))
+        plottable = compose(self.name, self)
+        plot_args = plot_args.with_defaults(name=self.name)
+        plot_args = plottable.prepare_plot_args(plot_args)
+        assert plot_args.graph, plot_args
+
+        ## Operations don't need another name visible.
+        #
+        if is_user_label:
+            del plot_args.graph.graph["graphviz.label"]
+        plot_args = plot_args._replace(plottable=self)
+
+        return plot_args
