@@ -23,8 +23,6 @@ from functools import partial
 from multiprocessing import Value
 from typing import Optional
 
-from boltons.iterutils import first
-
 _debug_env_var = os.environ.get("GRAPHTIK_DEBUG")
 _debug: ContextVar[Optional[bool]] = ContextVar(
     "debug",
@@ -34,6 +32,9 @@ _abort: ContextVar[Optional[bool]] = ContextVar(
     "abort", default=Value(ctypes.c_bool, lock=False)
 )
 _skip_evictions: ContextVar[Optional[bool]] = ContextVar("skip_evictions", default=None)
+_layered_solution: ContextVar[Optional[bool]] = ContextVar(
+    "layered_solution", default=None
+)
 _execution_pool: ContextVar[Optional["Pool"]] = ContextVar(
     "execution_pool", default=None
 )
@@ -79,14 +80,16 @@ When true, increase details on string-representation of network objects and erro
 
 :param enabled:
     - ``None, False, string(0, false, off, no)``: Disabled
-    - ``1``: Enable ALL ``DEBUG_XXX``
-    - integers: Enable respective ``DEBUG_XXX`` bit-field constants
+    - TODO: ``1``: Enable ALL ``DEBUG_XXX``
+    - TODO: integers: Enable respective ``DEBUG_XXX`` bit-field constants
     - anything else: Enable ALL ``DEBUG_XXX``
 
 Affected behavior:
 
 .. debug-behavior-start
 
++ :meth:`FnOp.compute()` raise any matching-needs errors immediately;
++ :meth:`FnOp.compute()` prints out full given-inputs (not just their keys);
 + net objects print details recursively;
 + plotted SVG diagrams include style-provenance as tooltips;
 + Sphinx extension also saves the original DOT file next to each image
@@ -136,7 +139,7 @@ is_skip_evictions = partial(_getter, _skip_evictions)
 """see :func:`set_skip_evictions()`"""
 set_skip_evictions = partial(_tristate_set, _skip_evictions)
 """
-When true, disable globally :term:`evictions`, to keep all intermediate solution values, ...
+When true, disable globally :term:`eviction`\\s, to keep all intermediate solution values, ...
 
 regardless of asked outputs.
 
@@ -145,6 +148,31 @@ regardless of asked outputs.
 """
 
 
+solution_layered = partial(_tristate_armed, _layered_solution)
+"""
+Like :func:`set_layered_solution()` as a context-manager, resetting back to old value.
+
+.. seealso:: disclaimer about context-managers the top of this :mod:`.config` module.
+"""
+is_layered_solution = partial(_getter, _layered_solution)
+"""see :func:`set_layered_solution()`"""
+set_layered_solution = partial(_tristate_set, _layered_solution)
+"""
+whether to store operation results into separate :term:`solution layer`
+
+:param enable:
+    - if None (default), results are layered only if there are NO :term:`jsonp` dependencies
+      in the network.
+    - When True(False), always keep(don't keep) results in a separate layer for each operation,
+      regardless of any *jsonp* dependencies.
+
+    It overrides any param given when executing a pipeline or a plan.
+
+:return:
+    a "reset" token (see :meth:`.ContextVar.set`)
+"""
+
+
 @contextmanager
 def execution_pool_plugged(pool: "Optional[Pool]"):
     """
