@@ -23,6 +23,7 @@ from collections import abc, namedtuple
 from contextlib import contextmanager
 from contextvars import ContextVar
 from functools import partial
+from itertools import chain
 from typing import (
     Any,
     Callable,
@@ -36,6 +37,7 @@ from typing import (
 )
 
 import jinja2
+from jinja2.filters import do_truncate
 import networkx as nx
 import pydot
 from boltons.iterutils import default_enter, default_exit, get_path, remap
@@ -49,7 +51,7 @@ from .config import (
     is_parallel_tasks,
     is_reschedule_operations,
 )
-from .modifiers import is_sfx, is_sfxed
+from .modifier import is_sfx, is_sfxed
 
 log = logging.getLogger(__name__)
 
@@ -161,7 +163,7 @@ def _monkey_patch_for_jupyter(pydot):
 
         def parse_dot_data(s):
             """Patched to fix pydot/pydot#171 by letting ex bubble-up."""
-            global top_graphs
+            global top_graphs  # pylint: disable=variable-not-defined-globally
 
             top_graphs = list()
             graphparser = dot_parser.graph_definition()
@@ -175,13 +177,16 @@ def _monkey_patch_for_jupyter(pydot):
 _monkey_patch_for_jupyter(pydot)
 
 
-def _is_class_value_in_list(lst, cls, value):
-    return any(isinstance(i, cls) and i == value for i in lst)
-
-
-def _merge_conditions(*conds):
-    """combines conditions as a choice in binary range, eg, 2 conds --> [0, 3]"""
-    return sum(int(bool(c)) << i for i, c in enumerate(conds))
+def is_nx_node_dependent(graph, nx_node):
+    """Return true if node's edges are not :term:`subdoc` only. """
+    return any(
+        1
+        for _src, _dst, subdoc in chain(
+            graph.in_edges(nx_node, data="subdoc"),
+            graph.out_edges(nx_node, data="subdoc"),
+        )
+        if not subdoc
+    )
 
 
 # TODO: move to base.py, to reduce fan-in imports (and be frank with module diagram).
@@ -299,6 +304,21 @@ def _format_exception(ex):
         return f"{type(ex).__name__}: {ex}"
 
 
+@jinja2.environmentfilter
+def _reversing_truncate(
+    env, s, length=255, killwords=False, end="...", leeway=None, reverse=None
+) -> str:
+    """
+    Like :class:`jinja2.filters.do_truncate` but from the end (keeping the suffix).
+    """
+    if not reverse:
+        return do_truncate(env, s, length, killwords, end, leeway)
+
+    s = s[::-1]
+    ret = do_truncate(env, s, length, killwords, end, leeway)
+    return ret[::-1]
+
+
 def _make_jinja2_environment() -> jinja2.Environment:
     env = jinja2.Environment()
 
@@ -313,6 +333,7 @@ def _make_jinja2_environment() -> jinja2.Environment:
     )
     env.filters["hrefer"] = _drop_gt_lt
     env.filters["ex"] = _format_exception
+    env.filters["truncate"] = _reversing_truncate
 
     return env
 
@@ -390,7 +411,10 @@ class Theme:
     ##########
     ## VARIABLES
 
+    #: args for jinja2 patched `truncate` filter, above.
+    truncate_args = ((23, True), {"reverse": True})
     fill_color = "wheat"
+    subdoc_color = "#8B4513"  # SaddleBrown
     pruned_color = "#d3d3d3"  # LightGrey
     canceled_color = "#a9a9a9"  # DarkGray
     failed_color = "LightCoral"
@@ -417,7 +441,7 @@ class Theme:
     }
     #: styles per plot-type
     kw_graph_plottable_type = {
-        "FunctionalOperation": {},
+        "FnOp": {},
         "Pipeline": {},
         "Network": {},
         "ExecutionPlan": {},
@@ -433,15 +457,16 @@ class Theme:
 
     #: Reduce margins, since sideffects take a lot of space
     #: (default margin: x=0.11, y=0.055O)
-    kw_data = {"margin": "0.04,0.02"}
-    #: SHAPE change if with inputs/outputs,
-    #: see https://graphviz.gitlab.io/_pages/doc/info/shapes.html
-    kw_data_io_choice = {
-        0: {"shape": "rect"},  # no IO
-        1: {"shape": "invhouse"},  # Inp
-        2: {"shape": "house"},  # Out
-        3: {"shape": "hexagon"},  # Inp/Out
+    kw_data = {
+        "shape": "rect",
+        "margin": "0.04,0.02",
+        "label": make_template("{{ nx_item | truncate }}"),
     }
+    kw_data_inp = {}
+    kw_data_out = {}
+    kw_data_inp_only = {"shape": "invhouse"}
+    kw_data_out_only = {"shape": "house"}
+    kw_data_io = {"shape": "hexagon"}
     kw_data_sideffect = {
         "color": "blue",
         "fontcolor": "blue",
@@ -527,11 +552,36 @@ class Theme:
     #: below.
     op_badge_styles = {
         "badge_styles": {
-            "!": {"tooltip": "endured", "bgcolor": "#04277d", "color": "white"},
-            "?": {"tooltip": "rescheduled", "bgcolor": "#fc89ac", "color": "white"},
-            "|": {"tooltip": "parallel", "bgcolor": "#b1ce9a", "color": "white"},
-            "&": {"tooltip": "marshalled", "bgcolor": "#4e3165", "color": "white"},
-            "}": {"tooltip": "returns_dict", "bgcolor": "#cc5500", "color": "white",},
+            "!": {
+                "tooltip": "endured",
+                "bgcolor": "#04277d",
+                "color": "white",
+                "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-endured",
+            },
+            "?": {
+                "tooltip": "rescheduled",
+                "bgcolor": "#fc89ac",
+                "color": "white",
+                "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-partial-outputs",
+            },
+            "|": {
+                "tooltip": "parallel",
+                "bgcolor": "#b1ce9a",
+                "color": "white",
+                "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-parallel-execution",
+            },
+            "&": {
+                "tooltip": "marshalled",
+                "bgcolor": "#4e3165",
+                "color": "white",
+                "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-marshalling",
+            },
+            "}": {
+                "tooltip": "returns_dict",
+                "bgcolor": "#cc5500",
+                "color": "white",
+                "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-returns-dictionary",
+            },
         }
     }
     #: props of the HTML-Table label for Operations
@@ -539,6 +589,8 @@ class Theme:
         "op_name": lambda pa: pa.nx_item.name,
         "fn_name": lambda pa: pa.nx_item
         and func_name(pa.nx_item.fn, mod=1, fqdn=1, human=1),
+        "op_truncate": Ref("truncate_args"),
+        "fn_truncate": Ref("truncate_args"),
         "op_tooltip": make_op_tooltip,
         "fn_tooltip": make_fn_tooltip,
         "op_url": Ref("op_url", default=None),
@@ -566,7 +618,7 @@ class Theme:
                   } | xmlattr }}
                 >
                     {%- if fontcolor -%}<FONT COLOR="{{ fontcolor }}">{%- endif -%}
-                    {{- '<B>OP:</B> <I>%s</I>' % op_name |ee if op_name -}}
+                    {{- '<B>OP:</B> <I>%s</I>' % op_name | truncate(*op_truncate[0], **op_truncate[1]) | ee if op_name -}}
                     {%- if fontcolor -%}</FONT>{%- endif -%}
                 </TD>
                 <TD BORDER="1" SIDES="b">
@@ -574,7 +626,10 @@ class Theme:
                     <TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2">
                         <TR>
                         {%- for badge in badges -%}
-                            <TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="{{ badge_styles[badge].bgcolor }}" TITLE="{{ badge_styles[badge].tooltip | e }}" TARGET="_self"
+                            <TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="{{ badge_styles[badge].bgcolor
+                                }}" TITLE="{{ badge_styles[badge].tooltip | e
+                                }}" HREF="{{ badge_styles[badge].URL | hrefer | ee
+                                }}" TARGET="_blank"
                             ><FONT FACE="monospace" COLOR="{{ badge_styles[badge].color }}"><B>
                                 {{- badge | eee -}}
                             </B></FONT></TD>
@@ -596,7 +651,7 @@ class Theme:
                     {%- if fontcolor -%}
                     <FONT COLOR="{{ fontcolor }}">
                     {%- endif -%}
-                    <B>FN:</B> {{ fn_name | eee }}
+                    <B>FN:</B> {{ fn_name | truncate(*fn_truncate[0], **fn_truncate[1]) | eee }}
                     {%- if fontcolor -%}
                     </FONT>
                     {%- endif -%}
@@ -611,7 +666,9 @@ class Theme:
     ## EDGE
     ##
 
-    kw_edge = {}
+    kw_edge = {"tailport": "s", "headport": "n"}
+    kw_edge_tail_op = {}
+    kw_edge_head_op = {"arrowtail": "inv", "dir": "back"}
     kw_edge_optional = {"style": ["dashed"]}
     kw_edge_sideffect = {"color": "blue"}
     #: Added conditionally if `alias_of` found in edge-attrs.
@@ -621,13 +678,20 @@ class Theme:
             "<<I>(alias of)</I><BR/>{{ nx_attrs['alias_of'] | eee }}>"
         ),
     }
-    #: Rendered if ``fn_kwarg`` exists in `nx_attrs`.
-    kw_edge_mapping_fn_kwarg = {
+    #: Rendered if ``keyword`` exists in `nx_attrs`.
+    kw_edge_mapping_keyword = {
         "fontsize": 11,  # default: 14
-        "label": make_template(
-            "<<I>(mapped)</I><BR/>{{ nx_attrs['fn_kwarg'] | eee }}>"
-        ),
+        "fontname": "italic",
+        "label": make_template("<&gt;{{ nx_attrs['keyword'] | eee }}>"),
     }
+    kw_edge_subdoc = {
+        "color": Ref("subdoc_color"),
+        "arrowtail": "odot",
+        "dir": "back",
+        "tailport": "se",
+        "headport": "nw",
+    }
+
     kw_edge_pruned = {"color": Ref("pruned_color")}
     kw_edge_rescheduled = {"style": ["dashed"]}
     kw_edge_endured = {"style": ["dashed"]}
@@ -637,7 +701,12 @@ class Theme:
     ## Other
     ##
 
-    include_steps = False
+    #: When true, plot also :term:`execution steps`, linking operations and evictions
+    #: with green dotted lines labeled with numbers denoting the execution order.
+    show_steps = False
+    #: When true, plot also :term:`hierarchical data` nodes that
+    #: are not directly linked to operations.
+    show_chaindocs = False
     kw_step = {
         "style": "dotted",  # Note: Step styles are not *remerged*.`
         "color": Ref("steps_color"),
@@ -1041,9 +1110,15 @@ class Plotter:
         if plot_args.name:
             dot.set_name(as_identifier(plot_args.name))
 
+        hidden = set()
+
         ## NODES
         #
         for nx_node, data in graph.nodes.data(True):
+            if not theme.show_chaindocs and not is_nx_node_dependent(graph, nx_node):
+                hidden.add(nx_node)
+                continue
+
             plot_args = base_plot_args._replace(nx_item=nx_node, nx_attrs=data)
             dot_node = self._make_node(plot_args)
             plot_args = plot_args._replace(dot_item=dot_node)
@@ -1053,12 +1128,15 @@ class Plotter:
         ## EDGES
         #
         for src, dst, data in graph.edges.data(True):
+            if src in hidden or dst in hidden:
+                continue
+
             plot_args = base_plot_args._replace(nx_item=(src, dst), nx_attrs=data)
             dot.add_edge(self._make_edge(plot_args))
 
         ## Draw steps sequence, if it's worth it.
         #
-        if steps and theme.include_steps and len(steps) > 1:
+        if steps and theme.show_steps and len(steps) > 1:
             it1 = iter(steps)
             it2 = iter(steps)
             next(it2)
@@ -1103,7 +1181,7 @@ class Plotter:
 
         3. Set tooltips with the solution-values for data-nodes.
         """
-        theme = plot_args.theme
+        theme: Theme = plot_args.theme
         graph = plot_args.graph
         nx_node = plot_args.nx_item
         node_attrs = plot_args.nx_attrs
@@ -1120,12 +1198,18 @@ class Plotter:
             #
             styles.add("node-name", {"name": quote_node_id(nx_node)})
 
-            io_choice = _merge_conditions(
-                inputs and nx_node in inputs, outputs and nx_node in outputs
-            )
-            styles.add(
-                f"kw_data_io_choice: {io_choice}", theme.kw_data_io_choice[io_choice]
-            )
+            is_inp = inputs and nx_node in inputs
+            is_out = outputs and nx_node in outputs
+            if is_inp:
+                styles.add("kw_data_inp")
+                if not is_out:
+                    styles.add("kw_data_inp_only")
+            if is_out:
+                styles.add("kw_data_out")
+                if not is_inp:
+                    styles.add("kw_data_out_only")
+            if is_inp and is_out:
+                styles.add("kw_data_io")
 
             if is_sfx(nx_node):
                 styles.add("kw_data_sideffect")
@@ -1169,13 +1253,13 @@ class Plotter:
 
             ## Op-kind
             #
-            if first_solid(nx_node.rescheduled, is_reschedule_operations()):
+            if first_solid(is_reschedule_operations(), nx_node.rescheduled):
                 label_styles.add("kw_op_rescheduled")
-            if first_solid(nx_node.endured, is_endure_operations()):
+            if first_solid(is_endure_operations(), nx_node.endured):
                 label_styles.add("kw_op_endured")
-            if first_solid(nx_node.parallel, is_parallel_tasks()):
+            if first_solid(is_parallel_tasks(), nx_node.parallel):
                 label_styles.add("kw_op_parallel")
-            if first_solid(nx_node.marshalled, is_marshal_tasks()):
+            if first_solid(is_marshal_tasks(), nx_node.marshalled):
                 label_styles.add("kw_op_marshalled")
             if nx_node.returns_dict:
                 label_styles.add("kw_op_returns_dict")
@@ -1264,14 +1348,20 @@ class Plotter:
         styles = self._new_styles_stack(plot_args)
 
         styles.add("kw_edge")
+        if isinstance(src, Operation):
+            styles.add("kw_edge_tail_op")
+        if isinstance(dst, Operation):
+            styles.add("kw_edge_head_op")
         if edge_attrs.get("optional"):
             styles.add("kw_edge_optional")
         if edge_attrs.get("sideffect"):
             styles.add("kw_edge_sideffect")
         if edge_attrs.get("alias_of"):
             styles.add("kw_edge_alias")
-        if edge_attrs.get("fn_kwarg"):
-            styles.add("kw_edge_mapping_fn_kwarg")
+        if edge_attrs.get("keyword"):
+            styles.add("kw_edge_mapping_keyword")
+        if edge_attrs.get("subdoc"):
+            styles.add("kw_edge_subdoc")
 
         if getattr(src, "rescheduled", None):
             styles.add("kw_edge_rescheduled")
@@ -1454,10 +1544,13 @@ class Plotter:
             label="Graphtik Legend %(ver)s";
 
             Dependencies   [shape=plaintext fontsize=16 fontname="bold italic"];
-            Dependencies -> dependency [tooltip="Compulsory: from src Operation --> dst `provides` OR src `needs` --> dst Operation."
-                        URL="%(arch_url)s#term-dependency"];
-            dependency [color=invis];
-            dependency -> optional     [style=dashed];
+            Dependencies -> needs [dir=back arrowtail=inv
+                                tooltip="Compulsory dependency from src `needs` data --> dst Operation."
+                                URL="%(arch_url)s#term-needs"];
+            needs [color=invis];
+            needs -> provides     [tooltip="Compulsory dependency from src Operation --> dst `provides` data"];
+            provides [color=invis];
+            provides -> optional     [style=dashed];
             optional    [color=invis label="optional\npartial out"
                         tooltip="Target operation may run without source `need` OR source operation may not `provide` target data."
                         URL="%(arch_url)s#term-needs"];
@@ -1469,8 +1562,13 @@ class Plotter:
             broken      [color=invis
                         tooltip="Target data was not `provided` by source operation due to failure / partial-outs."
                         URL="%(arch_url)s#term-partial outputs"];
-            broken   -> sequence        [color="%(steps_color)s" penwidth=4 style=dotted
+            broken   -> subdoc [color="%(subdoc_color)s" arrowtail=odot dir=back]
+            subdoc      [color=invis
+                        tooltip="Parent --> child document, accessed with `json pointer` dependencies."
+                        URL="%(arch_url)s#term-jsonp"];
+            subdoc  -> sequence [color="%(steps_color)s" penwidth=4 style=dotted
                                         arrowhead=vee label=1 fontcolor="%(steps_color)s"];
+
             sequence    [color=invis penwidth=4 label="execution\nsequence"
                         tooltip="Sequence of execution steps."
                         URL="%(arch_url)s#term-execution-steps"];
