@@ -1,4 +1,4 @@
-# Copyright 2016, Yahoo Inc.
+# Copyright 2020-2020, Kostis Anagnostopoulos;
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 import functools as fnt
 import itertools as itt
@@ -8,7 +8,7 @@ from unittest.mock import MagicMock
 
 import pytest
 
-from graphtik import base, network, operation, pipeline
+from graphtik import base, planning, operation, pipeline
 from graphtik.execution import ExecutionPlan, Solution, _OpTask
 from graphtik.base import Operation
 from graphtik.pipeline import Pipeline
