@@ -14,9 +14,11 @@ from graphtik import (
     NO_RESULT,
     NO_RESULT_BUT_SFX,
     compose,
+    keyword,
     operation,
     optional,
     sfx,
+    sfxed,
     vararg,
     varargs,
 )
@@ -26,10 +28,10 @@ from graphtik.config import (
     tasks_in_parallel,
     tasks_marshalled,
 )
-from graphtik.modifiers import dep_renamed
-from graphtik.network import yield_ops
-from graphtik.op import (
-    FunctionalOperation,
+from graphtik.modifier import dep_renamed
+from graphtik.planning import yield_ops
+from graphtik.fnop import (
+    FnOp,
     Operation,
     as_renames,
     reparse_operation_data,
@@ -63,11 +65,11 @@ def test_repr_smoke(opname, opneeds, opprovides):
 def test_repr_returns_dict():
     assert (
         str(operation(lambda: None, name="", returns_dict=True))
-        == "FunctionalOperation(name='', fn{}='<lambda>')"
+        == "FnOp(name='', fn{}='<lambda>')"
     )
     assert (
         str(operation(lambda: None, name="myname"))
-        == "FunctionalOperation(name='myname', fn='<lambda>')"
+        == "FnOp(name='myname', fn='<lambda>')"
     )
 
 
@@ -78,30 +80,24 @@ def test_auto_func_name():
 
 def test_builder_pattern():
     opb = operation()
-    assert not isinstance(opb, FunctionalOperation)
-    assert str(opb).startswith("<function FunctionalOperation.withset at")
+    assert not isinstance(opb, FnOp)
+    assert str(opb).startswith("<function FnOp.withset at")
 
-    assert str(opb(sum)) == "FunctionalOperation(name='sum', fn='sum')"
+    assert str(opb(sum)) == "FnOp(name='sum', fn='sum')"
 
-    assert str(opb(name="K")) == "FunctionalOperation(name='K', fn=None)"
+    assert str(opb(name="K")) == "FnOp(name='K', fn='identity_fn')"
 
     op = opb.withset(needs=["a", "b"])
-    assert isinstance(op, FunctionalOperation)
-    assert str(op) == "FunctionalOperation(name=None, needs=['a', 'b'], fn=None)"
+    assert isinstance(op, FnOp)
+    assert str(op) == "FnOp(name=None, needs=['a', 'b'], fn=None)"
     with pytest.raises(ValueError, match=f"Operation must have a callable `fn` and"):
         op.compute({})
 
     op = op.withset(provides="SUM", fn=sum, name=None)
-    assert (
-        str(op)
-        == "FunctionalOperation(name='sum', needs=['a', 'b'], provides=['SUM'], fn='sum')"
-    )
+    assert str(op) == "FnOp(name='sum', needs=['a', 'b'], provides=['SUM'], fn='sum')"
 
     op = op.withset(provides="s", needs="a", fn=str)
-    assert (
-        str(op)
-        == "FunctionalOperation(name='sum', needs=['a'], provides=['s'], fn='str')"
-    )
+    assert str(op) == "FnOp(name='sum', needs=['a'], provides=['s'], fn='str')"
 
 
 @pytest.mark.parametrize(
@@ -110,18 +106,18 @@ def test_builder_pattern():
         ((None, None, None), (None, (), ())),
         ## Check name
         (("_", "a", ("A",)), ("_", ("a",), ("A",))),
-        (((), ("a",), None), ((), ("a",), ())),
-        ((("a",), "a", "b"), (("a",), ("a",), ("b",))),
-        ((("a",), None, None), (("a",), (), ())),
+        (("", ("a",), None), ("", ("a",), ())),
+        (("a", "a", "b"), ("a", ("a",), ("b",))),
+        (("a", None, None), ("a", (), ())),
         ## Check needs
-        (((), (), None), ((), (), ())),
-        (((), [], None), ((), [], ())),
+        (("", (), None), ("", (), ())),
+        (("", [], None), ("", [], ())),
         (("", object(), None), ValueError("Cannot tuple-ize needs")),
         (("", [None], None), TypeError("All `needs` must be str")),
         (("", [()], None), TypeError("All `needs` must be str")),
         ## Check provides
-        (((), "a", ()), ((), ("a",), ())),
-        (((), "a", []), ((), ("a",), [])),
+        (("", "a", ()), ("", ("a",), ())),
+        (("", "a", []), ("", ("a",), [])),
         (("", "a", object()), ValueError("Cannot tuple-ize provides")),
         (("", "a", (None,)), TypeError("All `provides` must be str")),
         (("", "a", [()]), TypeError("All `provides` must be str")),
@@ -154,9 +150,9 @@ def test_func_op_validation(opargs, exp):
 def test_func_op_init(args, kw, exp):
     if isinstance(exp, Exception):
         with pytest.raises(type(exp), match=str(exp)):
-            FunctionalOperation(str, "", *args, **kw)
+            FnOp(str, "", *args, **kw)
     else:
-        FunctionalOperation(str, "", *args, **kw)
+        FnOp(str, "", *args, **kw)
 
 
 @pytest.mark.parametrize(
@@ -416,37 +412,37 @@ def test_reschedule_unknown_dict_outs(caplog):
 
 def test_rescheduled_op_repr():
     op = operation(str, name="t", provides=["a"], rescheduled=True)
-    assert str(op) == "FunctionalOperation?(name='t', provides=['a'], fn='str')"
+    assert str(op) == "FnOp?(name='t', provides=['a'], fn='str')"
 
 
 def test_endured_op_repr():
     op = operation(str, name="t", provides=["a"], endured=True)
-    assert str(op) == "FunctionalOperation!(name='t', provides=['a'], fn='str')"
+    assert str(op) == "FnOp!(name='t', provides=['a'], fn='str')"
 
 
 def test_endured_rescheduled_op_repr():
     op = operation(str, name="t", rescheduled=1, endured=1)
-    assert str(op) == "FunctionalOperation!?(name='t', fn='str')"
+    assert str(op) == "FnOp!?(name='t', fn='str')"
 
 
 def test_parallel_op_repr():
     op = operation(str, name="t", provides=["a"], parallel=True)
-    assert str(op) == "FunctionalOperation|(name='t', provides=['a'], fn='str')"
+    assert str(op) == "FnOp|(name='t', provides=['a'], fn='str')"
 
 
 def test_marshalled_op_repr():
     op = operation(str, name="t", provides=["a"], marshalled=True)
-    assert str(op) == ("FunctionalOperation&(name='t', provides=['a'], fn='str')")
+    assert str(op) == ("FnOp&(name='t', provides=['a'], fn='str')")
 
 
 def test_marshalled_parallel_op_repr():
     op = operation(str, name="t", parallel=1, marshalled=1)
-    assert str(op) == "FunctionalOperation|&(name='t', fn='str')"
+    assert str(op) == "FnOp|&(name='t', fn='str')"
 
 
 def test_ALL_op_repr():
     op = operation(str, name="t", rescheduled=1, endured=1, parallel=1, marshalled=1)
-    assert str(op) == "FunctionalOperation!?|&(name='t', fn='str')"
+    assert str(op) == "FnOp!?|&(name='t', fn='str')"
 
 
 def test_CONFIG_op_repr():
@@ -454,9 +450,9 @@ def test_CONFIG_op_repr():
         1
     ), tasks_marshalled(1):
         op = operation(str, name="t")
-        assert str(op) == "FunctionalOperation!?|&(name='t', fn='str')"
+        assert str(op) == "FnOp!?|&(name='t', fn='str')"
     op = operation(str, name="t", rescheduled=0, endured=0, parallel=0, marshalled=0)
-    assert str(op) == "FunctionalOperation(name='t', fn='str')"
+    assert str(op) == "FnOp(name='t', fn='str')"
 
 
 def test_reschedule_outputs():
@@ -537,7 +533,9 @@ def test_pipeline_conveys_attr_to_ops(attr, value):
     kw = {attr: value}
     _opsattrs(compose("1", operation(str), **kw).net.graph, attr, value)
     _opsattrs(
-        compose("2", operation(str, name=1), operation(str, name=2), **kw).net.graph,
+        compose(
+            "2", operation(str, name="1"), operation(str, name="2"), **kw
+        ).net.graph,
         attr,
         value,
     )
@@ -547,7 +545,7 @@ def test_pipeline_conveys_attr_to_ops(attr, value):
     "op_fact",
     [
         lambda: Operation,
-        lambda: FunctionalOperation,
+        lambda: FnOp,
         lambda: operation(str),
         lambda: operation(lambda: None),
     ],
@@ -562,6 +560,25 @@ def test_serialize_op(op_fact, ser_method):
     assert ser_method(op) == op
 
 
+def test_serialize_pipeline(samplenet, ser_method):
+    def eq(pipe1, pipe2):
+        return pipe1.name == pipe2.name and pipe1.ops == pipe2.ops
+
+    assert eq(ser_method(samplenet), samplenet)
+
+
+def test_non_op_given():
+    with pytest.raises(
+        TypeError, match=r"(?s)Received x1 non-Operation instances.+\[1\].+\['b', 'a'\]"
+    ):
+        compose("", operation(None, name="b"), 1, operation(None, name="a"))
+    with pytest.raises(
+        TypeError,
+        match=r"(?s)Received x1 non-Operation instances.+'noop'.+\['b', 'a'\]",
+    ):
+        compose("", "noop", operation(None, name="b"), operation(None, name="a"))
+
+
 def test_op_rename():
     op = operation(
         str, name="op1", needs=sfx("a"), provides=["a", sfx("b")], aliases=[("a", "b")],
@@ -576,11 +593,39 @@ def test_op_rename():
     got = str(ren)
     assert got == (
         """
-    FunctionalOperation(name='PP.op1', needs=[sfx('PP.a')], provides=['PP.a', sfx('PP.b')], aliases=[('PP.a', 'PP.b')], fn='str')
+    FnOp(name='PP.op1', needs=[sfx('PP.a')], provides=['PP.a', sfx('PP.b')], aliases=[('PP.a', 'PP.b')], fn='str')
         """.strip()
     )
 
 
+def test_op_rename_parts():
+    op = operation(
+        str,
+        name="op1",
+        needs=[sfx("a/b"), "/a/b"],
+        provides=["b/c", sfxed("d/e/f", "k/l")],
+        aliases=[("b/c", "/b/t")],
+    )
+
+    def renamer(na):
+        if na.name and na.typ.endswith(".jsonpart"):
+            return f"PP.{na.name}"
+
+    ren = op.withset(renamer=renamer)
+    got = str(ren)
+    print(got)
+    assert got == re.sub(
+        r"[\n ]+",  # collapse all space-chars into a single space
+        " ",
+        """
+        FnOp(name='op1',
+            needs=[sfx('a/b'), '/PP.a/PP.b'($)],
+            provides=['PP.b/PP.c'($), sfxed('PP.d/PP.e/PP.f'($), 'k/l')],
+            aliases=[('PP.b/PP.c'($), '/PP.b/PP.t'($))], fn='str')
+        """.strip(),
+    )
+
+
 def test_pipe_rename():
     pipe = compose(
         "t",
@@ -611,8 +656,8 @@ def test_pipe_rename():
         r"[\n ]+",  # collapse all space-chars into a single space
         " ",
         """
-        [FunctionalOperation(name='PP.op1', needs=[sfx('PP.a')], fn='str'),
-         FunctionalOperation(name='PP.op2', needs=[sfx('PP.a')], provides=['PP.a', sfx('PP.b')],
+        [FnOp(name='PP.op1', needs=[sfx('PP.a')], fn='str'),
+         FnOp(name='PP.op2', needs=[sfx('PP.a')], provides=['PP.a', sfx('PP.b')],
          aliases=[('PP.a', 'PP.b')], fn='str')]
         """.strip(),
     )
@@ -627,8 +672,42 @@ def test_pipe_rename():
         r"[\n ]+",  # collapse all space-chars into a single space
         " ",
         """
-        [FunctionalOperation(name='OP1', needs=[sfx('a')], fn='str'),
-         FunctionalOperation(name='op2', needs=[sfx('a')], provides=['a'(?), sfx('b')],
+        [FnOp(name='OP1', needs=[sfx('a')], fn='str'),
+         FnOp(name='op2', needs=[sfx('a')], provides=['a'(?), sfx('b')],
          aliases=[('a'(?), 'B')], fn='str')]
         """.strip(),
     )
+
+
+def test_conveyor_identity_fn():
+    op = operation(name="copy values", needs="a")()
+    assert not op.fn
+    op = operation(None, needs="a", provides="A")
+    assert not op.fn
+
+    op = operation(None, name="a", needs="a", provides="A")
+    assert op.fn
+    assert op(a=5) == {"A": 5}
+
+    op = operation(name="a", needs=["a", "b"], provides=["A", "B"])()
+    assert op.compute({"a": 5, "b": 6}) == {"A": 5, "B": 6}
+
+    op = operation(name="a", needs=["a", keyword("b")], provides=["A", "B"])()
+    assert op(a=55, b=66) == {"A": 55, "B": 66}
+
+    op = operation(
+        fn=None,
+        name="a",
+        needs=[optional("a"), vararg("b"), "c"],
+        # positional, vararg, keyword, optional
+        provides=["C", "B", "A"],
+    )
+    assert op(c=7, a=5, b=6) == {"A": 5, "B": 6, "C": 7}
+
+    # TODO: when `varargs` work for Outputs.
+    # op = operation(
+    #     name="a", needs=[sfxed("a", 's'), optional=1), sfxed_varargs("b", 's'), "c"],
+    #     # vararg, keyword, optional
+    #     provides=["C", "B", "A", varargs(sadasa"b", 's')]
+    # )()
+    # assert op(c=7, a=5, b=[6]) == {"A": 5, "B1": [6], "B2", "C": 7}
